@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-embed-single.c,v 1.16 2003/11/22 08:08:49 chpe Exp $
+ *  $Id: ephy-embed-single.c,v 1.19 2004/02/20 11:49:42 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -25,7 +25,7 @@
 #include "ephy-embed-single.h"
 #include "ephy-marshal.h"
 
-static void ephy_embed_single_class_init (gpointer g_class);
+static void ephy_embed_single_iface_init (gpointer g_class);
 
 GType
 ephy_embed_single_get_type (void)
@@ -36,8 +36,8 @@ ephy_embed_single_get_type (void)
 	{
 		static const GTypeInfo our_info =
 		{
-			sizeof (EphyEmbedSingleClass),
-			ephy_embed_single_class_init,
+			sizeof (EphyEmbedSingleIface),
+			ephy_embed_single_iface_init,
 			NULL,
 		};
 
@@ -51,7 +51,7 @@ ephy_embed_single_get_type (void)
 }
 
 static void
-ephy_embed_single_class_init (gpointer g_class)
+ephy_embed_single_iface_init (gpointer g_class)
 {
 	static gboolean initialised = FALSE;
 
@@ -71,7 +71,7 @@ ephy_embed_single_class_init (gpointer g_class)
 	g_signal_new ("handle_content",
 		      EPHY_TYPE_EMBED_SINGLE,
 		      G_SIGNAL_RUN_LAST,
-		      G_STRUCT_OFFSET (EphyEmbedSingleClass, handle_content),
+		      G_STRUCT_OFFSET (EphyEmbedSingleIface, handle_content),
 		      g_signal_accumulator_true_handled, NULL,
 		      ephy_marshal_BOOLEAN__STRING_STRING,
 		      G_TYPE_BOOLEAN,
@@ -92,8 +92,21 @@ ephy_embed_single_class_init (gpointer g_class)
 void
 ephy_embed_single_clear_cache (EphyEmbedSingle *single)
 {
-	EphyEmbedSingleClass *klass = EPHY_EMBED_SINGLE_GET_CLASS (single);
-	klass->clear_cache (single);
+	EphyEmbedSingleIface *iface = EPHY_EMBED_SINGLE_GET_IFACE (single);
+	iface->clear_cache (single);
+}
+
+/**
+ * ephy_embed_single_clear_auth_cache:
+ * @single: the #EphyEmbedSingle
+ * 
+ * Clears the mozilla http authentication cache.
+ **/
+void
+ephy_embed_single_clear_auth_cache (EphyEmbedSingle *single)
+{
+	EphyEmbedSingleIface *iface = EPHY_EMBED_SINGLE_GET_IFACE (single);
+	iface->clear_auth_cache (single);
 }
 
 /**
@@ -107,8 +120,8 @@ void
 ephy_embed_single_set_offline_mode (EphyEmbedSingle *single,
 				    gboolean offline)
 {
-	EphyEmbedSingleClass *klass = EPHY_EMBED_SINGLE_GET_CLASS (single);
-	klass->set_offline_mode (single, offline);
+	EphyEmbedSingleIface *iface = EPHY_EMBED_SINGLE_GET_IFACE (single);
+	iface->set_offline_mode (single, offline);
 }
 
 /**
@@ -122,8 +135,8 @@ void
 ephy_embed_single_load_proxy_autoconf (EphyEmbedSingle *single,
 				       const char* address)
 {
-	EphyEmbedSingleClass *klass = EPHY_EMBED_SINGLE_GET_CLASS (single);
-	klass->load_proxy_autoconf (single, address);
+	EphyEmbedSingleIface *iface = EPHY_EMBED_SINGLE_GET_IFACE (single);
+	iface->load_proxy_autoconf (single, address);
 }
 
 /**
@@ -140,6 +153,6 @@ GList *
 ephy_embed_single_get_font_list (EphyEmbedSingle *single,
 				 const char *langGroup)
 {
-	EphyEmbedSingleClass *klass = EPHY_EMBED_SINGLE_GET_CLASS (single);
-	return klass->get_font_list (single, langGroup);
+	EphyEmbedSingleIface *iface = EPHY_EMBED_SINGLE_GET_IFACE (single);
+	return iface->get_font_list (single, langGroup);
 }
