@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: FilePicker.cpp,v 1.24 2004/02/15 17:01:21 chpe Exp $
+ *  $Id: FilePicker.cpp,v 1.26 2004/02/29 14:24:40 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -63,7 +63,7 @@ NS_IMPL_ISUPPORTS1(GFilePicker, nsIFilePicker)
 
 GFilePicker::GFilePicker()
 {
-	LOG ("GFilePicker constructor")
+	LOG ("GFilePicker ctor (%p)", this)
 
 	mDialog = EPHY_FILE_CHOOSER (g_object_new (EPHY_TYPE_FILE_CHOOSER, NULL));
 
@@ -76,7 +76,7 @@ GFilePicker::GFilePicker()
 
 GFilePicker::~GFilePicker()
 {
-	LOG ("GFilePicker destructor")
+	LOG ("GFilePicker dtor (%p)", this)
 
 	if (mDialog)
 	{
@@ -104,9 +104,21 @@ NS_IMETHODIMP GFilePicker::Init(nsIDOMWindowInternal *parent, const PRUnichar *t
 
 	switch (mode)
 	{
-		case nsIFilePicker::modeOpen:
 		case nsIFilePicker::modeGetFolder:
+			gtk_file_chooser_set_action (GTK_FILE_CHOOSER (mDialog),
+						     GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER);
+			
+			gtk_dialog_add_buttons (GTK_DIALOG (mDialog),
+						GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+						GTK_STOCK_OPEN, EPHY_RESPONSE_OPEN,
+						NULL);
+			gtk_dialog_set_default_response (GTK_DIALOG (mDialog), EPHY_RESPONSE_OPEN);
+			break;
+
 		case nsIFilePicker::modeOpenMultiple:
+			gtk_file_chooser_set_select_multiple (GTK_FILE_CHOOSER (mDialog), TRUE);
+			/* fallthrough */			
+		case nsIFilePicker::modeOpen:
 			gtk_file_chooser_set_action (GTK_FILE_CHOOSER (mDialog),
 						     GTK_FILE_CHOOSER_ACTION_OPEN);
 
@@ -117,6 +129,7 @@ NS_IMETHODIMP GFilePicker::Init(nsIDOMWindowInternal *parent, const PRUnichar *t
 			gtk_dialog_set_default_response (GTK_DIALOG (mDialog), EPHY_RESPONSE_OPEN);
 
 			break;
+
 		case nsIFilePicker::modeSave:
 			gtk_file_chooser_set_action (GTK_FILE_CHOOSER (mDialog),
 						     GTK_FILE_CHOOSER_ACTION_SAVE);
@@ -132,12 +145,6 @@ NS_IMETHODIMP GFilePicker::Init(nsIDOMWindowInternal *parent, const PRUnichar *t
 			break;
 	}
 
-	gtk_file_chooser_set_select_multiple (GTK_FILE_CHOOSER (mDialog),
-					      mode == nsIFilePicker::modeOpenMultiple);
-
-	gtk_file_chooser_set_folder_mode (GTK_FILE_CHOOSER (mDialog),
-					  mode == nsIFilePicker::modeGetFolder);
-
 	return NS_OK;
 }
 
