@@ -14,15 +14,16 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *
+ *  $Id: ephy-gui.h,v 1.9 2004/02/21 19:59:01 chpe Exp $
  */
 
 #ifndef EPHY_GUI_H
 #define EPHY_GUI_H
 
-/* system includes */
 #include <gtk/gtk.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
-#include <gnome.h>
+#include <gdk/gdkkeysyms.h>
 
 G_BEGIN_DECLS
 
@@ -38,6 +39,7 @@ gboolean	ephy_gui_confirm_overwrite_file	         (GtkWidget *parent,
 void		ephy_gui_help				 (GtkWindow *parent,
 							  const char *file_name,
 							  const char *link_id);
+
 G_END_DECLS
 
 #endif
