@@ -1,6 +1,7 @@
 /*
  *  Copyright (C) 2002 Christophe Fergeau
- *  Copyright (C) 2003 Christian Persch
+ *  Copyright (C) 2003 Marco Pesenti Gritti
+ *  Copyright (C) 2003, 2004 Christian Persch
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,11 +17,11 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-notebook.c,v 1.18 2003/08/06 09:47:55 chpe Exp $
+ *  $Id: ephy-notebook.c,v 1.32 2004/03/06 19:02:38 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #include "ephy-notebook.h"
@@ -34,11 +35,20 @@
 #include "ephy-shell.h"
 #include "ephy-debug.h"
 #include "ephy-favicon-cache.h"
-#include "ephy-event-box.h"
 
-#include <gtk/gtk.h>
 #include <glib-object.h>
-#include <bonobo/bonobo-i18n.h>
+#include <gtk/gtkeventbox.h>
+#include <gtk/gtknotebook.h>
+#include <gtk/gtkhbox.h>
+#include <gtk/gtklabel.h>
+#include <gtk/gtkwidget.h>
+#include <gtk/gtktooltips.h>
+#include <gtk/gtkmain.h>
+#include <gtk/gtkstock.h>
+#include <gtk/gtkimage.h>
+#include <gtk/gtkbutton.h>
+#include <gtk/gtkiconfactory.h>
+#include <glib/gi18n.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
 
 #define AFTER_ALL_TABS -1
@@ -46,6 +56,8 @@
 #define TAB_MIN_SIZE 60
 #define TAB_NB_MAX    8
 
+#define EPHY_NOTEBOOK_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_NOTEBOOK, EphyNotebookPrivate))
+
 struct EphyNotebookPrivate
 {
 	GList *focused_pages;
@@ -56,8 +68,6 @@ struct EphyNotebookPrivate
 	gint x_start, y_start;
 	gboolean drag_in_progress;
 	gboolean show_tabs;
-	EphyNotebook *src_notebook;
-	gint src_page;
 };
 
 static void ephy_notebook_init           (EphyNotebook *notebook);
@@ -65,11 +75,10 @@ static void ephy_notebook_class_init     (EphyNotebookClass *klass);
 static void ephy_notebook_finalize       (GObject *object);
 static void move_tab_to_another_notebook (EphyNotebook *src,
 			                  EphyNotebook *dest,
-					  gint dest_page);
+					  int dest_position);
 
 /* Local variables */
 static GdkCursor *cursor = NULL;
-static GList *notebooks  = NULL;
 
 static GtkTargetEntry url_drag_types [] = 
 {
@@ -84,13 +93,14 @@ enum
 	TAB_REMOVED,
 	TABS_REORDERED,
 	TAB_DETACHED,
+	TAB_DELETE,
 	LAST_SIGNAL
 };
 
-static GObjectClass *parent_class = NULL;
-
 static guint signals[LAST_SIGNAL] = { 0 };
 
+static GObjectClass *parent_class = NULL;
+
 GType
 ephy_notebook_get_type (void)
 {
@@ -134,39 +144,51 @@ ephy_notebook_class_init (EphyNotebookClass *klass)
 			      G_SIGNAL_RUN_FIRST,
 			      G_STRUCT_OFFSET (EphyNotebookClass, tab_added),
 			      NULL, NULL,
-			      ephy_marshal_VOID__OBJECT,
+			      g_cclosure_marshal_VOID__OBJECT,
 			      G_TYPE_NONE,
 			      1,
-			      GTK_TYPE_WIDGET);
+			      EPHY_TYPE_TAB);
 	signals[TAB_REMOVED] =
 		g_signal_new ("tab_removed",
 			      G_OBJECT_CLASS_TYPE (object_class),
 			      G_SIGNAL_RUN_FIRST,
 			      G_STRUCT_OFFSET (EphyNotebookClass, tab_removed),
 			      NULL, NULL,
-			      ephy_marshal_VOID__OBJECT,
+			      g_cclosure_marshal_VOID__OBJECT,
 			      G_TYPE_NONE,
 			      1,
-			      GTK_TYPE_WIDGET);
+			      EPHY_TYPE_TAB);
 	signals[TAB_DETACHED] =
 		g_signal_new ("tab_detached",
 			      G_OBJECT_CLASS_TYPE (object_class),
 			      G_SIGNAL_RUN_FIRST,
 			      G_STRUCT_OFFSET (EphyNotebookClass, tab_detached),
 			      NULL, NULL,
-			      ephy_marshal_VOID__OBJECT,
+			      g_cclosure_marshal_VOID__OBJECT,
 			      G_TYPE_NONE,
 			      1,
-			      GTK_TYPE_WIDGET);
+			      EPHY_TYPE_TAB);
 	signals[TABS_REORDERED] =
 		g_signal_new ("tabs_reordered",
 			      G_OBJECT_CLASS_TYPE (object_class),
 			      G_SIGNAL_RUN_FIRST,
 			      G_STRUCT_OFFSET (EphyNotebookClass, tabs_reordered),
 			      NULL, NULL,
-			      ephy_marshal_VOID__VOID,
+			      g_cclosure_marshal_VOID__VOID,
 			      G_TYPE_NONE,
 			      0);
+	signals[TAB_DELETE] =
+		g_signal_new ("tab_delete",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (EphyNotebookClass, tab_delete),
+			      g_signal_accumulator_true_handled, NULL,
+			      ephy_marshal_BOOLEAN__OBJECT,
+			      G_TYPE_BOOLEAN,
+			      1,
+			      EPHY_TYPE_TAB);
+
+	g_type_class_add_private (object_class, sizeof(EphyNotebookPrivate));
 }
 
 static gboolean
@@ -193,36 +215,36 @@ is_in_notebook_window (EphyNotebook *notebook,
 static EphyNotebook *
 find_notebook_at_pointer (gint abs_x, gint abs_y)
 {
-	GList *l;
+	GdkWindow *win_at_pointer, *toplevel_win;
+	gpointer toplevel = NULL;
 	gint x, y;
-	GdkWindow *win_at_pointer = gdk_window_at_pointer (&x, &y);
-	GdkWindow *parent_at_pointer = NULL;
 
+	win_at_pointer = gdk_window_at_pointer (&x, &y);
 	if (win_at_pointer == NULL)
 	{
 		/* We are outside all windows containing a notebook */
 		return NULL;
 	}
 
-	gdk_window_get_toplevel (win_at_pointer);
-	/* When we are in the notebook event window, win_at_pointer will be
-	   this event window, and the toplevel window we are interested in
-	   will be its parent
-	*/
-	parent_at_pointer = gdk_window_get_parent (win_at_pointer);
+	toplevel_win = gdk_window_get_toplevel (win_at_pointer);
 
-	for (l = notebooks; l != NULL; l = l->next)
+	/* get the GtkWidget which owns the toplevel GdkWindow */
+	gdk_window_get_user_data (toplevel_win, &toplevel);
+
+	/* toplevel should be an EphyWindow */
+	if (toplevel != NULL && EPHY_IS_WINDOW (toplevel))
 	{
-		EphyNotebook *nb = EPHY_NOTEBOOK (l->data);
-		GdkWindow *win = GTK_WIDGET (nb)->window;
+		EphyNotebook *notebook;
 
-		win = gdk_window_get_toplevel (win);
-		if (((win == win_at_pointer) || (win == parent_at_pointer))
-		    && is_in_notebook_window (nb, abs_x, abs_y))
+		notebook = EPHY_NOTEBOOK (ephy_window_get_notebook
+					    (EPHY_WINDOW (toplevel)));
+
+		if (is_in_notebook_window (notebook, abs_x, abs_y))
 		{
-			return nb;
+			return notebook;
 		}
 	}
+
 	return NULL;
 }
 
@@ -323,26 +345,28 @@ tab_label_set_size (GtkWidget *window, GtkWidget *label)
 static void
 tab_label_size_request_cb (GtkWidget *window,
 			   GtkRequisition *requisition,
-			   GtkWidget *child)
+			   GtkWidget *tab)
 {
 	GtkWidget *hbox;
 	GtkWidget *nb;
 
-	nb = child->parent;
+	nb = tab->parent;
 
 	hbox = gtk_notebook_get_tab_label (GTK_NOTEBOOK (nb),
-					   child);
+					   tab);
 	tab_label_set_size (window, hbox);
 }
 
-
 void
-ephy_notebook_move_page (EphyNotebook *src, EphyNotebook *dest,
-			 GtkWidget *src_page,  gint dest_page)
+ephy_notebook_move_tab (EphyNotebook *src,
+			EphyNotebook *dest,
+			EphyTab *tab,
+			int dest_position)
 {
 	if (dest == NULL || src == dest)
 	{
-		gtk_notebook_reorder_child (GTK_NOTEBOOK (src), src_page, dest_page);
+		gtk_notebook_reorder_child
+			(GTK_NOTEBOOK (src), GTK_WIDGET (tab), dest_position);
 		
 		if (src->priv->drag_in_progress == FALSE)
 		{
@@ -351,23 +375,18 @@ ephy_notebook_move_page (EphyNotebook *src, EphyNotebook *dest,
 	}
 	else
 	{
-		/* make sure the tab isn't destroyed while we move its embed */
-		g_object_ref (G_OBJECT (src_page));
-		ephy_notebook_remove_page (EPHY_NOTEBOOK (src), src_page);
-		ephy_notebook_insert_page (EPHY_NOTEBOOK (dest), src_page,
-					  dest_page, TRUE);
-		g_object_unref (G_OBJECT (src_page));
+		/* make sure the tab isn't destroyed while we move it */
+		g_object_ref (tab);
+		ephy_notebook_remove_tab (src, tab);
+		ephy_notebook_insert_tab (dest, tab, dest_position, TRUE);
+		g_object_unref (tab);
 	}
 }
 
 static void
-drag_start (EphyNotebook *notebook,
-	    EphyNotebook *src_notebook,
-	    gint src_page)
+drag_start (EphyNotebook *notebook)
 {
 	notebook->priv->drag_in_progress = TRUE;
-	notebook->priv->src_notebook = src_notebook;
-	notebook->priv->src_page = src_page;
 
 	/* get a new cursor, if necessary */
 	if (!cursor) cursor = gdk_cursor_new (GDK_FLEUR);
@@ -391,8 +410,6 @@ drag_stop (EphyNotebook *notebook)
 	}
 
 	notebook->priv->drag_in_progress = FALSE;
-	notebook->priv->src_notebook = NULL;
-	notebook->priv->src_page = -1;
 	if (notebook->priv->motion_notify_handler_id != 0)
 	{
 		g_signal_handler_disconnect (G_OBJECT (notebook),
@@ -405,19 +422,22 @@ drag_stop (EphyNotebook *notebook)
  * here, instead we do it on drag_stop
  */
 static void
-move_tab (EphyNotebook *notebook, gint dest_page_num)
+move_tab (EphyNotebook *notebook,
+	  int dest_position)
 {
 	gint cur_page_num;
 
 	cur_page_num = gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook));
 
-	if (dest_page_num != cur_page_num)
+	if (dest_position != cur_page_num)
 	{
-		GtkWidget *cur_page;
-		cur_page = gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook),
-						      cur_page_num);
-		ephy_notebook_move_page (EPHY_NOTEBOOK (notebook), NULL, cur_page,
-					 dest_page_num);
+		GtkWidget *cur_tab;
+		
+		cur_tab = gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook),
+						     cur_page_num);
+		ephy_notebook_move_tab (EPHY_NOTEBOOK (notebook), NULL,
+					EPHY_TAB (cur_tab),
+					dest_position);
 
 		/* Reset the list of newly opened tabs when moving tabs. */
 		g_list_free (notebook->priv->opened_tabs);
@@ -426,30 +446,27 @@ move_tab (EphyNotebook *notebook, gint dest_page_num)
 }
 
 static gboolean
-motion_notify_cb (EphyNotebook *notebook, GdkEventMotion *event,
+motion_notify_cb (EphyNotebook *notebook,
+		  GdkEventMotion *event,
 		  gpointer data)
 {
 	EphyNotebook *dest;
 	gint page_num;
 	gint result;
 
-	/* If the notebook only has one tab, we don't want to do
-	 * anything since ephy can't handle empty notebooks
-	 */
-	if (g_list_length (GTK_NOTEBOOK (notebook)->children) <= 1) {
-		return FALSE;
-	}
-
-	if ((notebook->priv->drag_in_progress == FALSE)
-	    && (gtk_drag_check_threshold (GTK_WIDGET (notebook),
-					  notebook->priv->x_start,
-					  notebook->priv->y_start,
-					  event->x_root, event->y_root)))
+	if (notebook->priv->drag_in_progress == FALSE)
 	{
-		gint cur_page;
-
-		cur_page = gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook));
-		drag_start (notebook, notebook, cur_page);
+		if (gtk_drag_check_threshold (GTK_WIDGET (notebook),
+					      notebook->priv->x_start,
+					      notebook->priv->y_start,
+					      event->x_root, event->y_root))
+		{
+			drag_start (notebook);
+		}
+		else
+		{
+			return FALSE;
+		}
 	}
 
 	result = find_notebook_and_tab_at_pos ((gint)event->x_root,
@@ -474,32 +491,33 @@ motion_notify_cb (EphyNotebook *notebook, GdkEventMotion *event,
 }
 
 static void
-move_tab_to_another_notebook(EphyNotebook *src,
-			     EphyNotebook *dest, gint dest_page)
+move_tab_to_another_notebook (EphyNotebook *src,
+			      EphyNotebook *dest,
+			      int dest_position)
 {
-	GtkWidget *child;
-	gint cur_page;
+	EphyTab *tab;
+	int cur_page;
 
 	/* This is getting tricky, the tab was dragged in a notebook
 	 * in another window of the same app, we move the tab
 	 * to that new notebook, and let this notebook handle the
 	 * drag
 	*/
-	g_assert (dest != NULL);
+	g_assert (EPHY_IS_NOTEBOOK (dest));
 	g_assert (dest != src);
 
-	/* Move the widgets (tab label and tab content) to the new
-	 * notebook
-	 */
 	cur_page = gtk_notebook_get_current_page (GTK_NOTEBOOK (src));
-	child    = gtk_notebook_get_nth_page (GTK_NOTEBOOK (src), cur_page);
-	ephy_notebook_move_page (src, dest, child, dest_page);
+	tab = EPHY_TAB (gtk_notebook_get_nth_page (GTK_NOTEBOOK (src), cur_page));
 
-	/* "Give" drag handling to the new notebook */
-	drag_start (dest, src->priv->src_notebook, src->priv->src_page);
+	/* stop drag in origin window */
 	drag_stop (src);
 	gtk_grab_remove (GTK_WIDGET (src));
 
+	ephy_notebook_move_tab (src, dest, tab, dest_position);
+
+	/* start drag handling in dest notebook */
+	drag_start (dest);
+
 	dest->priv->motion_notify_handler_id =
 		g_signal_connect (G_OBJECT (dest),
 				  "motion-notify-event",
@@ -507,9 +525,9 @@ move_tab_to_another_notebook(EphyNotebook *src,
 				  NULL);
 }
 
-/* Callbacks */
 static gboolean
-button_release_cb (EphyNotebook *notebook, GdkEventButton *event,
+button_release_cb (EphyNotebook *notebook,
+		   GdkEventButton *event,
 		   gpointer data)
 {
 	if (notebook->priv->drag_in_progress)
@@ -522,7 +540,8 @@ button_release_cb (EphyNotebook *notebook, GdkEventButton *event,
 		cur_page = gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook),
 						      cur_page_num);
 
-		if (!is_in_notebook_window (notebook, event->x_root, event->y_root))
+		if (!is_in_notebook_window (notebook, event->x_root, event->y_root)
+		    && gtk_notebook_get_n_pages (GTK_NOTEBOOK (notebook)) > 1)
 		{
 			/* Tab was detached */
 			g_signal_emit (G_OBJECT (notebook),
@@ -541,7 +560,6 @@ button_release_cb (EphyNotebook *notebook, GdkEventButton *event,
 	return FALSE;
 }
 
-
 static gboolean
 button_press_cb (EphyNotebook *notebook,
 		 GdkEventButton *event,
@@ -573,14 +591,14 @@ button_press_cb (EphyNotebook *notebook,
 GtkWidget *
 ephy_notebook_new (void)
 {
-	return GTK_WIDGET (g_object_new (EPHY_NOTEBOOK_TYPE, NULL));
+	return GTK_WIDGET (g_object_new (EPHY_TYPE_NOTEBOOK, NULL));
 }
 
 static void
 ephy_notebook_switch_page_cb (GtkNotebook *notebook,
-                             GtkNotebookPage *page,
-                             guint page_num,
-                             gpointer data)
+                              GtkNotebookPage *page,
+                              guint page_num,
+                              gpointer data)
 {
 	EphyNotebook *nb = EPHY_NOTEBOOK (notebook);
 	GtkWidget *child;
@@ -608,11 +626,10 @@ ephy_notebook_switch_page_cb (GtkNotebook *notebook,
 static void
 notebook_drag_data_received_cb (GtkWidget* widget, GdkDragContext *context,
 				gint x, gint y, GtkSelectionData *selection_data,
-				guint info, guint time, GtkWidget *child)
+				guint info, guint time, EphyTab *tab)
 {
 	EphyEmbed *embed = NULL;
 	EphyWindow *window = NULL;
-	EphyTab *tab = NULL;
 	GtkWidget *toplevel;
 	GList *uris = NULL, *l;
 	GnomeVFSURI *uri;
@@ -622,6 +639,8 @@ notebook_drag_data_received_cb (GtkWidget* widget, GdkDragContext *context,
 
 	g_signal_stop_emission_by_name (widget, "drag_data_received");
 
+	if (eel_gconf_get_boolean (CONF_LOCKDOWN_DISABLE_ARBITRARY_URL)) return;
+
 	if (selection_data->length <= 0 || selection_data->data == NULL) return;
 
 	if (selection_data->target == gdk_atom_intern (EPHY_DND_URL_TYPE, FALSE))
@@ -647,13 +666,13 @@ notebook_drag_data_received_cb (GtkWidget* widget, GdkDragContext *context,
 	if (uris == NULL) return;
 
 	toplevel = gtk_widget_get_toplevel (widget);
-	g_return_if_fail (IS_EPHY_WINDOW (toplevel));
+	g_return_if_fail (EPHY_IS_WINDOW (toplevel));
 	window = EPHY_WINDOW (toplevel);
 
-	if (child)
+	if (tab != NULL);
 	{
-		embed = EPHY_EMBED (child);
-		tab = EPHY_TAB (g_object_get_data (G_OBJECT (embed), "EphyTab"));
+		embed = ephy_tab_get_embed (tab);
+		g_return_if_fail (EPHY_IS_EMBED (embed));
 	}
 
 	l = uris;
@@ -703,7 +722,7 @@ update_tabs_visibility (EphyNotebook *nb, gboolean before_inserting)
 
 	if (before_inserting) num++;
 
-	show_tabs = (eel_gconf_get_boolean (CONF_TABS_TABBED) || num > 1) &&
+	show_tabs = (eel_gconf_get_boolean (CONF_ALWAYS_SHOW_TABS_BAR) || num > 1) &&
 		    nb->priv->show_tabs == TRUE;
 
 	gtk_notebook_set_show_tabs (GTK_NOTEBOOK (nb), show_tabs);
@@ -721,7 +740,11 @@ tabs_visibility_notifier (GConfClient *client,
 static void
 ephy_notebook_init (EphyNotebook *notebook)
 {
-	notebook->priv = g_new (EphyNotebookPrivate, 1);
+	notebook->priv = EPHY_NOTEBOOK_GET_PRIVATE (notebook);
+
+	gtk_notebook_set_scrollable (GTK_NOTEBOOK (notebook), TRUE);
+	gtk_notebook_set_show_border (GTK_NOTEBOOK (notebook), FALSE);
+	gtk_notebook_set_show_tabs (GTK_NOTEBOOK (notebook), FALSE);
 
 	notebook->priv->title_tips = gtk_tooltips_new ();
 	g_object_ref (G_OBJECT (notebook->priv->title_tips));
@@ -729,14 +752,10 @@ ephy_notebook_init (EphyNotebook *notebook)
 
 	notebook->priv->drag_in_progress = FALSE;
 	notebook->priv->motion_notify_handler_id = 0;
-	notebook->priv->src_notebook = NULL;
-	notebook->priv->src_page = -1;
 	notebook->priv->focused_pages = NULL;
 	notebook->priv->opened_tabs = NULL;
 	notebook->priv->show_tabs = TRUE;
 
-	notebooks = g_list_append (notebooks, notebook);
-
 	g_signal_connect (notebook, "button-press-event",
 			  (GCallback)button_press_cb, NULL);
 	g_signal_connect (notebook, "button-release-event",
@@ -757,7 +776,7 @@ ephy_notebook_init (EphyNotebook *notebook)
                            GDK_ACTION_MOVE | GDK_ACTION_COPY);
 
 	notebook->priv->tabs_vis_notifier_id = eel_gconf_notification_add
-		(CONF_TABS_TABBED,
+		(CONF_ALWAYS_SHOW_TABS_BAR,
 		 (GConfClientNotifyFunc)tabs_visibility_notifier, notebook);
 }
 
@@ -766,8 +785,6 @@ ephy_notebook_finalize (GObject *object)
 {
 	EphyNotebook *notebook = EPHY_NOTEBOOK (object);
 
-	notebooks = g_list_remove (notebooks, notebook);
-
 	eel_gconf_notification_remove (notebook->priv->tabs_vis_notifier_id);
 
 	if (notebook->priv->focused_pages)
@@ -777,9 +794,9 @@ ephy_notebook_finalize (GObject *object)
 	g_list_free (notebook->priv->opened_tabs);
 	g_object_unref (notebook->priv->title_tips);
 
-	g_free (notebook->priv);
-
 	LOG ("EphyNotebook finalised %p", object)
+
+	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 static void
@@ -856,16 +873,22 @@ sync_label (EphyTab *tab, GParamSpec *pspec, GtkWidget *proxy)
 }
 
 static void
-close_button_clicked_cb (GtkWidget *widget, GtkWidget *child)
+close_button_clicked_cb (GtkWidget *widget, GtkWidget *tab)
 {
 	EphyNotebook *notebook;
+	gboolean inhibited = FALSE;
 
-	notebook = EPHY_NOTEBOOK (gtk_widget_get_parent (child));
-	ephy_notebook_remove_page (notebook, child);
+	notebook = EPHY_NOTEBOOK (gtk_widget_get_parent (tab));
+	g_signal_emit (G_OBJECT (notebook), signals[TAB_DELETE], 0, tab, &inhibited);
+
+	if (inhibited == FALSE)
+	{
+		ephy_notebook_remove_tab (notebook, EPHY_TAB (tab));
+	}
 }
 
 static GtkWidget *
-build_tab_label (EphyNotebook *nb, GtkWidget *child)
+build_tab_label (EphyNotebook *nb, EphyTab *tab)
 {
 	GtkWidget *label, *hbox, *close_button, *image;
 	int h, w;
@@ -905,7 +928,8 @@ build_tab_label (EphyNotebook *nb, GtkWidget *child)
 	gtk_box_pack_start (GTK_BOX (hbox), icon, FALSE, FALSE, 0);
 
 	/* setup label */
-	label_ebox = ephy_event_box_new ();
+	label_ebox = gtk_event_box_new ();
+	gtk_event_box_set_visible_window (GTK_EVENT_BOX (label_ebox), FALSE);
         label = gtk_label_new ("");
 	gtk_misc_set_alignment (GTK_MISC (label), 0.00, 0.5);
         gtk_misc_set_padding (GTK_MISC (label), 4, 0);
@@ -915,7 +939,7 @@ build_tab_label (EphyNotebook *nb, GtkWidget *child)
 	tab_label_set_size (GTK_WIDGET (window), hbox);
 
 	closure = g_cclosure_new (G_CALLBACK (tab_label_size_request_cb),
-				  child, NULL);
+				  tab, NULL);
 	g_object_watch_closure (G_OBJECT (label), closure);
 	g_signal_connect_closure_by_id (G_OBJECT (window),
 		g_signal_lookup ("size_request",
@@ -929,7 +953,7 @@ build_tab_label (EphyNotebook *nb, GtkWidget *child)
 
 	g_signal_connect (G_OBJECT (close_button), "clicked",
                           G_CALLBACK (close_button_clicked_cb),
-                          child);
+                          tab);
 
 	gtk_widget_show (hbox);
 	gtk_widget_show (label_ebox);
@@ -955,19 +979,16 @@ ephy_notebook_set_show_tabs (EphyNotebook *nb, gboolean show_tabs)
 }
 
 void
-ephy_notebook_insert_page (EphyNotebook *nb,
-			   GtkWidget *child,
-			   int position,
-			   gboolean jump_to)
+ephy_notebook_insert_tab (EphyNotebook *nb,
+			  EphyTab *tab,
+			  int position,
+			  gboolean jump_to)
 {
-	EphyTab *tab;
 	GtkWidget *label;
 
-	g_return_if_fail (IS_EPHY_EMBED (child));
-	tab = EPHY_TAB (g_object_get_data (G_OBJECT (child), "EphyTab"));
-	g_return_if_fail (IS_EPHY_TAB (tab));
+	g_return_if_fail (EPHY_IS_TAB (tab));
 
-	label = build_tab_label (nb, child);
+	label = build_tab_label (nb, tab);
 
 	update_tabs_visibility (nb, TRUE);
 
@@ -990,16 +1011,16 @@ ephy_notebook_insert_page (EphyNotebook *nb,
 				    (GTK_NOTEBOOK (nb)) + 1;
 		}
 		nb->priv->opened_tabs =
-			g_list_append (nb->priv->opened_tabs, child);
+			g_list_append (nb->priv->opened_tabs, tab);
 	}
 
 
-	gtk_notebook_insert_page (GTK_NOTEBOOK (nb), child,
+	gtk_notebook_insert_page (GTK_NOTEBOOK (nb), GTK_WIDGET (tab),
 				  label, position);
 
 	/* Set up drag-and-drop target */
 	g_signal_connect (G_OBJECT(label), "drag_data_received",
-			  G_CALLBACK(notebook_drag_data_received_cb), child);
+			  G_CALLBACK(notebook_drag_data_received_cb), tab);
 	gtk_drag_dest_set (label, GTK_DEST_DEFAULT_ALL,
 			   url_drag_types,n_url_drag_types,
 			   GDK_ACTION_MOVE | GDK_ACTION_COPY);
@@ -1015,25 +1036,25 @@ ephy_notebook_insert_page (EphyNotebook *nb,
 	g_signal_connect_object (tab, "notify::load-status",
 				 G_CALLBACK (sync_load_status), label, 0);
 
+	g_signal_emit (G_OBJECT (nb), signals[TAB_ADDED], 0, tab);
+
 	if (jump_to)
 	{
 		gtk_notebook_set_current_page (GTK_NOTEBOOK (nb),
 					       position);
-		g_object_set_data (G_OBJECT (child), "jump_to",
+		g_object_set_data (G_OBJECT (tab), "jump_to",
 				   GINT_TO_POINTER (jump_to));
 	}
-
-	g_signal_emit (G_OBJECT (nb), signals[TAB_ADDED], 0, child);
 }
 
 static void
 smart_tab_switching_on_closure (EphyNotebook *nb,
-				GtkWidget *child)
+				EphyTab *tab)
 {
 	gboolean jump_to;
 
 	jump_to = GPOINTER_TO_INT (g_object_get_data
-				   (G_OBJECT (child), "jump_to"));
+				   (G_OBJECT (tab), "jump_to"));
 
 	if (!jump_to || !nb->priv->focused_pages)
 	{
@@ -1056,18 +1077,14 @@ smart_tab_switching_on_closure (EphyNotebook *nb,
 }
 
 void
-ephy_notebook_remove_page (EphyNotebook *nb,
-			  GtkWidget *child)
+ephy_notebook_remove_tab (EphyNotebook *nb,
+			  EphyTab *tab)
 {
 	int num, position, curr;
-	EphyTab *tab;
 	GtkWidget *label, *ebox;
 
-	g_return_if_fail (IS_EPHY_NOTEBOOK (nb));
-	g_return_if_fail (IS_EPHY_EMBED (child));
-
-	tab = EPHY_TAB (g_object_get_data (G_OBJECT (child), "EphyTab"));
-	g_return_if_fail (IS_EPHY_TAB (tab));
+	g_return_if_fail (EPHY_IS_NOTEBOOK (nb));
+	g_return_if_fail (EPHY_IS_TAB (tab));
 
 	num = gtk_notebook_get_n_pages (GTK_NOTEBOOK (nb));
 	if (num <= 1)
@@ -1080,39 +1097,39 @@ ephy_notebook_remove_page (EphyNotebook *nb,
 
 	/* Remove the page from the focused pages list */
 	nb->priv->focused_pages =  g_list_remove (nb->priv->focused_pages,
-						  child);
-	nb->priv->opened_tabs = g_list_remove (nb->priv->opened_tabs, child);
+						  tab);
+	nb->priv->opened_tabs = g_list_remove (nb->priv->opened_tabs, tab);
 
-	position = gtk_notebook_page_num (GTK_NOTEBOOK (nb), child);
+	position = gtk_notebook_page_num (GTK_NOTEBOOK (nb), GTK_WIDGET (tab));
 	curr = gtk_notebook_get_current_page (GTK_NOTEBOOK (nb));
 
 	if (position == curr)
 	{
-		smart_tab_switching_on_closure (nb, child);
+		smart_tab_switching_on_closure (nb, tab);
 	}
 
-	label = gtk_notebook_get_tab_label (GTK_NOTEBOOK (nb), child);
+	label = gtk_notebook_get_tab_label (GTK_NOTEBOOK (nb), GTK_WIDGET (tab));
 	ebox = GTK_WIDGET (g_object_get_data (G_OBJECT (label), "label-ebox"));
 	gtk_tooltips_set_tip (GTK_TOOLTIPS (nb->priv->title_tips), ebox, NULL, NULL);
 
-	g_signal_handlers_disconnect_by_func (label,
-					      G_CALLBACK (sync_icon), tab);
-	g_signal_handlers_disconnect_by_func (label,
-					      G_CALLBACK (sync_label), tab);
-	g_signal_handlers_disconnect_by_func (label,
-					      G_CALLBACK (sync_load_status), tab);
+	g_signal_handlers_disconnect_by_func (tab,
+					      G_CALLBACK (sync_icon), label);
+	g_signal_handlers_disconnect_by_func (tab,
+					      G_CALLBACK (sync_label), label);
+	g_signal_handlers_disconnect_by_func (tab,
+					      G_CALLBACK (sync_load_status), label);
 
 	/**
-	 * we ref the child so that it's still alive while the tabs_removed
+	 * we ref the tab so that it's still alive while the tabs_removed
 	 * signal is processed.
 	 */
-	g_object_ref (child);
+	g_object_ref (tab);
 
 	gtk_notebook_remove_page (GTK_NOTEBOOK (nb), position);
 
 	update_tabs_visibility (nb, FALSE);
 
-	g_signal_emit (G_OBJECT (nb), signals[TAB_REMOVED], 0, child);
+	g_signal_emit (G_OBJECT (nb), signals[TAB_REMOVED], 0, tab);
 
-	g_object_unref (child);
+	g_object_unref (tab);
 }
